/* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
/* 
 * Proxy Profiles Applet
 * Copyright (C) 2003 Carlos Morgado <chbm@gnome.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
 


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdlib.h>


#include <gnome.h>
#include <gtk/gtk.h>

#include <glade/glade.h>
#include <libgnomeui/gnome-window-icon.h>
#include <panel-applet-gconf.h>
#include <gconf/gconf-client.h>

#include "config.h"

typedef struct _proxy_properties proxy_properties;

struct _proxy_properties {
    gchar *name;
    gchar *http_host;
    guint16 http_port;
    gchar *https_host;
    guint16 https_port;
    gchar *ftp_host;
    guint16 ftp_port;
    gchar *socks_host;
    guint16 socks_port;
    gboolean use_auth;
    gchar *user;
    gchar *pass;
};

typedef struct _applet_properties applet_properties;

struct _applet_properties {
    gchar *version;
    gchar *selected;
    gboolean useproxy;
    GList *proxies;
};


typedef struct _ProxyApplet ProxyApplet;
struct _ProxyApplet {
    PanelApplet *applet;
    applet_properties properties; /* = { NULL }; */

    gchar* glade_file;
    GladeXML *xml;
    GtkWidget *toplevel;
    GtkWidget *editprof;
    GtkWidget *profnamediag;
    GtkWidget *aboutdiag;

    GtkWidget *evbox;
    GtkWidget *pixmap;
    GtkWidget *proflist;

    GtkWidget *prof_menu;

    GConfClient * gconfc;
};




static void
remove_proxy_from_config( proxy_properties *proxy );
static void
save_proxy( gpointer data, ProxyApplet *self );
static void
applet_create_proflist( ProxyApplet *applet );
static void
set_selected_profile( ProxyApplet *self );

show_error_dialog (ProxyApplet *self, gchar *mesg,...)
{
    char *tmp;
    va_list ap;
    GtkWidget *dialogWindow;
 
    va_start (ap,mesg);         
    tmp = g_strdup_vprintf (mesg,ap);
    dialogWindow = gnome_message_box_new (tmp, GNOME_MESSAGE_BOX_ERROR,
					  GNOME_STOCK_BUTTON_OK,NULL);
    gnome_dialog_run_and_close (GNOME_DIALOG (dialogWindow));
    g_free (tmp);
    va_end (ap);
}


const gchar *authors[] = { "Carlos Morgado <chbm@gnome.org>" };

void
display_about_dialog (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
{
    ProxyApplet *self = user_data;

    self->aboutdiag = 
	gnome_about_new ("Proxy Profiles Applet",
			 VERSION,
			 "(c) Carlos Morgado 2003",
			 _("An applet to manage GNOME network proxy profiles"),
			 authors,
			 NULL,
			 NULL,
			 NULL );

    gtk_widget_show_all (self->aboutdiag);
}


#define GCONFDIR "/apps/proxy-profiles-applet"

static gint
find_profile( gconstpointer a,
	      gconstpointer b) 
{
    return (a == b);
}

static gint
find_profile_name( gconstpointer a,
		   gconstpointer b) 
{
    proxy_properties *proxy = (proxy_properties *)a;
    return (strcmp( proxy->name, b ));
}

static void
free_proxy( proxy_properties *proxy )
{
    g_free(proxy->name);
    g_free(proxy->http_host);
    g_free(proxy->https_host);
    g_free(proxy->ftp_host);
    g_free(proxy->socks_host);
    g_free(proxy->user);
    g_free(proxy->pass);
}


static void
delete_profile_real( proxy_properties *proxy, ProxyApplet *self ) 
{
    remove_proxy_from_config(proxy); 
    free_proxy(proxy);
    self->properties.proxies = 
	g_list_remove( self->properties.proxies, proxy );
}


static void
profile_edit_fill( proxy_properties *proxy, ProxyApplet *self )
{
    GtkWidget *widget;

    widget = glade_xml_get_widget (self->xml, "profile_entry");
    gtk_entry_set_text( GTK_ENTRY(widget), proxy->name );

    widget = glade_xml_get_widget (self->xml, "http_entry");
    gtk_entry_set_text( GTK_ENTRY(widget), proxy->http_host );
    widget = glade_xml_get_widget (self->xml, "https_entry");
    gtk_entry_set_text( GTK_ENTRY(widget), proxy->https_host );
    widget = glade_xml_get_widget (self->xml, "ftp_entry");
    gtk_entry_set_text( GTK_ENTRY(widget), proxy->ftp_host );
    widget = glade_xml_get_widget (self->xml, "socks_entry");
    gtk_entry_set_text( GTK_ENTRY(widget), proxy->socks_host );
  
    widget = glade_xml_get_widget (self->xml, "http_p_spin");
    gtk_spin_button_set_value ( GTK_SPIN_BUTTON(widget), (gdouble)proxy->http_port );
    widget = glade_xml_get_widget (self->xml, "https_p_spin");
    gtk_spin_button_set_value ( GTK_SPIN_BUTTON(widget), (gdouble)proxy->https_port );
    widget = glade_xml_get_widget (self->xml, "ftp_p_spin");
    gtk_spin_button_set_value ( GTK_SPIN_BUTTON(widget), (gdouble)proxy->ftp_port );
    widget = glade_xml_get_widget (self->xml, "socks_p_spin");
    gtk_spin_button_set_value ( GTK_SPIN_BUTTON(widget), (gdouble)proxy->socks_port );

    widget = glade_xml_get_widget (self->xml, "auth_check");
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(widget), proxy->use_auth );
    widget = glade_xml_get_widget (self->xml, "user_entry");
    gtk_entry_set_text( GTK_ENTRY(widget), proxy->user );
    widget = glade_xml_get_widget (self->xml, "pass_entry");
    gtk_entry_set_text( GTK_ENTRY(widget), proxy->pass );

}

static void
cancel_profile_edit_cb( GtkWidget *widget, ProxyApplet *self )
{

    gtk_widget_hide(self->editprof);
  
}


static void
apply_profile_edit_cb( GtkWidget *widget, ProxyApplet *self )
{
    GtkWidget *w;
    const gchar *newname;
    GList *elem;
    proxy_properties *proxy;
    gboolean setsel = FALSE;

    proxy = g_object_get_data( G_OBJECT(widget), "proxy" );
  
    w = glade_xml_get_widget (self->xml, "profile_entry");
    newname = gtk_entry_get_text( GTK_ENTRY(w) );

    if( !strcmp( "", newname) ) {
	gtk_dialog_run (GTK_DIALOG(self->profnamediag));
	return;
    }
    g_object_set_data ( G_OBJECT(widget), "proxy", NULL );


    if (proxy) {
	if( !strcmp( self->properties.selected, proxy->name )) {
	    g_free(self->properties.selected);
	    self->properties.selected = g_strdup(newname);
	    setsel = TRUE;
	}
	delete_profile_real( proxy, self );    
    }
    proxy = g_new0( proxy_properties, 1 );

    proxy->name = g_strdup(newname);
  
    w = glade_xml_get_widget (self->xml, "http_entry");
    proxy->http_host = g_strdup( gtk_entry_get_text( GTK_ENTRY(w) ));
    w = glade_xml_get_widget (self->xml, "https_entry");
    proxy->https_host = g_strdup( gtk_entry_get_text( GTK_ENTRY(w) ));

  
    w = glade_xml_get_widget (self->xml, "ftp_entry");
    proxy->ftp_host = g_strdup( gtk_entry_get_text( GTK_ENTRY(w) ));
    w = glade_xml_get_widget (self->xml, "socks_entry");
    proxy->socks_host = g_strdup(gtk_entry_get_text( GTK_ENTRY(w) ));
  
    w = glade_xml_get_widget (self->xml, "http_p_spin");
    proxy->http_port =  gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON(w) ); 

    w = glade_xml_get_widget (self->xml, "https_p_spin");
    proxy->https_port = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON(w) );

    w = glade_xml_get_widget (self->xml, "ftp_p_spin");
    proxy->ftp_port = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON(w) );

    w = glade_xml_get_widget (self->xml, "socks_p_spin");
    proxy->socks_port = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON(w) );


    w = glade_xml_get_widget (self->xml, "auth_check");
    proxy->use_auth = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(w) );
  
    w = glade_xml_get_widget (self->xml, "user_entry");
    proxy->user = g_strdup( gtk_entry_get_text( GTK_ENTRY(w) ));

    w = glade_xml_get_widget (self->xml, "pass_entry");
    proxy->pass = g_strdup( gtk_entry_get_text( GTK_ENTRY(w) ));
 
    self->gconfc = gconf_client_get_default();
    save_proxy( proxy, self );
    g_object_unref (self->gconfc);
    self->properties.proxies = 
	g_list_append(self->properties.proxies, proxy ); /* humm */

    gtk_widget_hide(self->editprof);

    applet_create_proflist ( self );

    if(setsel)
	set_selected_profile(self);
}


static void
add_profile_cb( GtkWidget *widget, ProxyApplet *self )
{
    proxy_properties *proxy;
    GtkTreePath *path;
    GtkWidget *w;
    GList *elem;

    elem = g_list_find_custom( self->properties.proxies, 
			       "default", 
			       find_profile_name );
  
    if (elem)
	profile_edit_fill( elem->data, self );
    w = glade_xml_get_widget (self->xml, "profile_entry");
    gtk_entry_set_text( GTK_ENTRY(w), "" );

    w = glade_xml_get_widget (self->xml, "apply_button");
    g_object_set_data ( G_OBJECT(w), "proxy", NULL );

    gtk_widget_show_all( self->editprof );

}


static void
delete_profile_cb( GtkWidget *widget, ProxyApplet *self )
{
    GtkTreePath *path;
    GtkTreeSelection * selection = 
	gtk_tree_view_get_selection (GTK_TREE_VIEW(self->proflist));
    GtkTreeModel *model = 
	gtk_tree_view_get_model (GTK_TREE_VIEW(self->proflist));
    GtkTreeIter iter;
    GList *elem;
    gchar *profile;

    if(!gtk_tree_selection_get_selected (selection, &model, &iter))
	return;
  
    gtk_tree_model_get ( model, &iter,
			 0, &profile, -1 );
  
    elem = g_list_find_custom( self->properties.proxies, 
			       profile, 
			       find_profile_name );

    delete_profile_real( elem->data, self );

    applet_create_proflist ( self );
}


static void
edit_profile_cb( GtkWidget *widget, ProxyApplet *self )
{
    GtkTreePath *path;
    GtkTreeSelection * selection = 
	gtk_tree_view_get_selection (GTK_TREE_VIEW(self->proflist));
    GtkTreeModel *model = 
	gtk_tree_view_get_model (GTK_TREE_VIEW(self->proflist));
    GtkTreeIter iter;
    gchar *profile;
    GList *elem;
    GtkWidget *w;

    if(!gtk_tree_selection_get_selected (selection, &model, &iter))
	return;

    gtk_tree_model_get ( model, &iter,
			 0, &profile, -1 );

    elem = g_list_find_custom( self->properties.proxies, 
			       profile, 
			       find_profile_name );

    profile_edit_fill( elem->data, self );
    w = glade_xml_get_widget (self->xml, "apply_button");
    g_object_set_data ( G_OBJECT(w), "proxy", elem->data );
  
    gtk_widget_show_all( self->editprof );
}


static void
remove_proxy_from_config( proxy_properties *proxy ) 
{
    GConfClient * gconfc = gconf_client_get_default();
    gchar *foo;
  
  
    foo = g_strdup_printf( "%s/%s",
			   GCONFDIR,
			   proxy->name );
  
    gconf_client_unset ( gconfc, foo, NULL );


    foo = g_strdup_printf( "%s/%s/name", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);


    foo = g_strdup_printf( "%s/%s/http_host", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL );
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/http_port", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);


    foo = g_strdup_printf( "%s/%s/https_host", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/https_port", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);

    foo = g_strdup_printf( "%s/%s/ftp_host", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/ftp_port", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);


    foo = g_strdup_printf( "%s/%s/socks_host", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/socks_port", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);

    foo = g_strdup_printf( "%s/%s/use_auth", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);

    foo = g_strdup_printf( "%s/%s/user", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/pass", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);

    foo = g_strdup_printf( "%s/%s", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_unset ( gconfc, 
			 foo , 
			 NULL);
    g_free(foo);


}

static void
set_selected_profile( ProxyApplet *self ) 
{

    GConfClient * gconfc = gconf_client_get_default();


    if(self->properties.useproxy) {
	proxy_properties *proxy;
	GList *elem =
	    g_list_find_custom( self->properties.proxies, 
				self->properties.selected,
				find_profile_name );
	proxy = elem->data;

	gconf_client_set_string (gconfc, 
				 "/system/http_proxy/host",
				 proxy->http_host,
				 NULL );    
	gconf_client_set_int (gconfc, 
			      "/system/http_proxy/port", 
			      proxy->http_port,			    
			      NULL );
	gconf_client_set_bool (gconfc, 
			       "/system/http_proxy/use_authentication", 
			       proxy->use_auth,
			       NULL );
	gconf_client_set_string (gconfc, 
				 "/system/http_proxy/authentication_user", 
				 proxy->user,
				 NULL );
	gconf_client_set_string (gconfc, 
				 "/system/http_proxy/authentication_password", 
				 proxy->pass,
				 NULL );
	gconf_client_set_string (gconfc, 
				 "/system/proxy/secure_host", 
				 proxy->https_host,
				 NULL );
	gconf_client_set_int (gconfc, 
			      "/system/proxy/secure_port", 
			      proxy->https_port, 
			      NULL );
	gconf_client_set_string (gconfc, 
				 "/system/proxy/ftp_host", 
				 proxy->ftp_host, 
				 NULL );
	gconf_client_set_int (gconfc, 
			      "/system/proxy/ftp_port", 
			      proxy->ftp_port, 
			      NULL );
	gconf_client_set_string (gconfc, 
				 "/system/proxy/socks_host", 
				 proxy->socks_host,
				 NULL );
	gconf_client_set_int (gconfc, 
			      "/system/proxy/socks_port", 
			      proxy->socks_port, 
			      NULL );
	gconf_client_set_bool( gconfc,
			       "/system/http_proxy/use_http_proxy",
			       TRUE,
			       NULL );
	gconf_client_set_string (gconfc, 
				 "/system/proxy/mode",
				 "manual",
				 NULL );
    } else {
	gconf_client_set_bool( gconfc,
			       "/system/http_proxy/use_http_proxy",
			       FALSE,
			       NULL );	
	gconf_client_set_string (gconfc, 
				 "/system/proxy/mode",
				 "none",
				 NULL );
    }   

    /* save our own prefs */
    gconf_client_set_string (gconfc, 
			     GCONFDIR "/selected",
			     self->properties.selected,
			     NULL );
    gconf_client_set_bool (gconfc, 
			   GCONFDIR "/use_proxy",
			   self->properties.useproxy,
			   NULL );

    g_object_unref(gconfc);
}

static void
save_proxy( gpointer data, ProxyApplet *self ) 
{
    proxy_properties *proxy = data;
    GConfClient * gconfc = self->gconfc;
    gchar *foo;

    foo = g_strdup_printf( "%s/%s/name", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_string ( gconfc, 
			      foo , 
			      proxy->name,
			      NULL);
    g_free(foo);


    foo = g_strdup_printf( "%s/%s/http_host", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_string ( gconfc, 
			      foo , 
			      proxy->http_host,
			      NULL);
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/http_port", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_int ( gconfc, 
			   foo , 
			   proxy->http_port,
			   NULL);
    g_free(foo);


    foo = g_strdup_printf( "%s/%s/https_host", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_string ( gconfc, 
			      foo , 
			      proxy->https_host,
			      NULL);
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/https_port", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_int ( gconfc, 
			   foo , 
			   proxy->https_port,
			   NULL);
    g_free(foo);

    foo = g_strdup_printf( "%s/%s/ftp_host", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_string ( gconfc, 
			      foo , 
			      proxy->ftp_host,
			      NULL);
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/ftp_port", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_int ( gconfc, 
			   foo , 
			   proxy->ftp_port,
			   NULL);
    g_free(foo);


    foo = g_strdup_printf( "%s/%s/socks_host", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_string ( gconfc, 
			      foo , 
			      proxy->socks_host,
			      NULL);
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/socks_port", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_int ( gconfc, 
			   foo , 
			   proxy->socks_port,
			   NULL);
    g_free(foo);

    foo = g_strdup_printf( "%s/%s/use_auth", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_bool ( gconfc, 
			    foo , 
			    proxy->use_auth,
			    NULL);
    g_free(foo);

    foo = g_strdup_printf( "%s/%s/user", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_string ( gconfc, 
			      foo , 
			      proxy->user,
			      NULL);
    g_free(foo);
    foo = g_strdup_printf( "%s/%s/pass", 
			   GCONFDIR,
			   proxy->name );
    gconf_client_set_string ( gconfc, 
			      foo , 
			      proxy->pass,
			      NULL);
    g_free(foo);



  
}

static void
save_properties (ProxyApplet *self)
{
    GConfClient * gconfc = gconf_client_get_default();
  
    gconf_client_set_string ( gconfc, 
			      GCONFDIR "/version", 
			      self->properties.version,
			      NULL);
    gconf_client_set_string ( gconfc, 
			      GCONFDIR "/selected", 
			      self->properties.selected,
			      NULL);
    gconf_client_set_bool ( gconfc, 
			    GCONFDIR "/use_proxy", 
			    self->properties.useproxy,
			    NULL);
  
    g_list_foreach( self->properties.proxies, (GFunc)save_proxy, self );
}


static void
load_proxy( gpointer data, ProxyApplet *self ) 
{
    gchar *path = (char *)data;
    proxy_properties *proxy;
    gchar *foo;
    GConfClient * gconfc = gconf_client_get_default();

    proxy = g_new (proxy_properties, 1);

    proxy->name = g_strdup( path + strlen(GCONFDIR) + 1 );

    foo = g_strdup_printf ("%s/http_host", path);
    proxy->http_host = 
	gconf_client_get_string (gconfc, foo, NULL );
    g_free(foo);
    foo = g_strdup_printf ("%s/http_port", path);  
    proxy->http_port = 
	gconf_client_get_int (gconfc, foo, NULL );
    g_free(foo);

    foo = g_strdup_printf ("%s/https_host", path);
    proxy->https_host = 
	gconf_client_get_string (gconfc, foo, NULL );
    g_free(foo);
    foo = g_strdup_printf ("%s/https_port", path);
    proxy->https_port = 
	gconf_client_get_int (gconfc, foo, NULL);
    g_free(foo);
 
    foo = g_strdup_printf ("%s/ftp_host", path);
    proxy->ftp_host = 
	gconf_client_get_string (gconfc, foo, NULL );
    g_free(foo);
    foo = g_strdup_printf ("%s/ftp_port", path);
    proxy->ftp_port = 
	gconf_client_get_int (gconfc, foo, NULL );
    g_free(foo);

    foo = g_strdup_printf ("%s/socks_host", path);
    proxy->socks_host = 
	gconf_client_get_string (gconfc, foo, NULL );
    g_free(foo);
    foo = g_strdup_printf ("%s/socks_port", path);
    proxy->socks_port = 
	gconf_client_get_int (gconfc, foo, NULL );
    g_free(foo);

    foo = g_strdup_printf ("%s/use_auth", path);
    proxy->use_auth = 
	gconf_client_get_bool (gconfc, foo, NULL );
    g_free(foo);
    foo = g_strdup_printf ("%s/user", path);
    proxy->user = 
	gconf_client_get_string (gconfc, foo, NULL );
    g_free(foo);
    foo = g_strdup_printf ("%s/pass", path);    
    proxy->pass = 
	gconf_client_get_string (gconfc, foo, NULL );
    g_free(foo);

    self->properties.proxies = 
	g_list_append(self->properties.proxies, proxy );
}


static void
load_properties (ProxyApplet *self)
{

    GConfClient * gconfc = gconf_client_get_default();

    self->properties.proxies = NULL;
    self->gconfc = gconfc;

    self->properties.version = 
	gconf_client_get_string ( gconfc, GCONFDIR "/version", NULL);
    if(!self->properties.version) {
	/* untouched ! */
	proxy_properties *proxy;
    
	self->properties.version = g_strdup("0.1");

	proxy = g_new(proxy_properties, 1);

	proxy->name = g_strdup("default");
	proxy->http_host = 
	    gconf_client_get_string (gconfc, "/system/http_proxy/host", NULL );
	proxy->http_port = 
	    gconf_client_get_int (gconfc, "/system/http_proxy/port", NULL );
	proxy->use_auth = 
	    gconf_client_get_bool (gconfc, "/system/http_proxy/use_authentication", NULL );
	proxy->user = 
	    gconf_client_get_string (gconfc, "/system/http_proxy/authentication_user", NULL );
	proxy->pass = 
	    gconf_client_get_string (gconfc, "/system/http_proxy/authentication_password", NULL );
	proxy->https_host = 
	    gconf_client_get_string (gconfc, "/system/proxy/secure_host", NULL );
	proxy->https_port = 
	    gconf_client_get_int (gconfc, "/system/proxy/secure_port", NULL );
	proxy->ftp_host = 
	    gconf_client_get_string (gconfc, "/system/proxy/ftp_host", NULL );
	proxy->ftp_port = 
	    gconf_client_get_int (gconfc, "/system/proxy/ftp_port", NULL );
	proxy->socks_host = 
	    gconf_client_get_string (gconfc, "/system/proxy/socks_host", NULL );
	proxy->socks_port = 
	    gconf_client_get_int (gconfc, "/system/proxy/socks_port", NULL );
    
	self->properties.proxies = 
	    g_list_append(self->properties.proxies, proxy );

	self->properties.selected = g_strdup("default");
	self->properties.useproxy = 
	    gconf_client_get_bool( gconfc, 
				   "/system/proxy/use_http_proxy",
				   NULL );

	save_properties(self);

    } else {
	self->properties.version =
	    gconf_client_get_string ( gconfc, 
				      GCONFDIR "/version", 
				      NULL);
	self->properties.selected = 
	    gconf_client_get_string ( gconfc, 
				      GCONFDIR "/selected", 
				      NULL);
	self->properties.useproxy = 
	    gconf_client_get_bool ( gconfc, 
				    GCONFDIR "/use_proxy", 
				    NULL);
    
	GSList *dirlist = 
	    gconf_client_all_dirs (gconfc, 
				   GCONFDIR,
				   NULL);
    
	g_slist_foreach( dirlist, (GFunc)load_proxy, self );

	set_selected_profile (self);
    }

    g_object_unref(gconfc);
    self->gconfc = NULL;
}

static void
close_properties_dialog( GtkWidget *widget, ProxyApplet *applet ) 
{
    gtk_widget_hide(applet->toplevel);
}

static void
insert_profile( gpointer data, gpointer user_data) {
    static proxy_properties *proxy;
    GtkTreeIter iter;
    GtkListStore *store;

    proxy = data;
    store = user_data;

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, proxy->name, -1);
}

static void
applet_create_proflist( ProxyApplet *applet ) 
{
    GtkListStore *store = gtk_list_store_new (1, G_TYPE_STRING);

    applet->properties.proxies = 
	g_list_first (applet->properties.proxies); 
    g_list_foreach ( applet->properties.proxies,
		     (GFunc)insert_profile,
		     store );

    gtk_tree_view_set_model (GTK_TREE_VIEW(applet->proflist),
			     GTK_TREE_MODEL(store) );
    
    gtk_widget_show_all(GTK_WIDGET(applet->proflist));
}

static void
display_properties_dialog (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
{
    ProxyApplet *applet = user_data;

    applet_create_proflist( applet );
    gtk_widget_show_all( applet->toplevel );
}

static const char proxy_menu_xml [] =
    "<popup name=\"button3\">\n"
    "   <menuitem name=\"Edit Profiles\" verb=\"EditProxies\" _label=\"Edit Profiles\"\n"
    "             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
    "   <menuitem name=\"About...\" verb=\"AboutDialog\" _label=\"About\"\n"
    "             pixtype=\"stock\" pixname=\"gtk-about\"/>\n"
    "</popup>\n";

static const BonoboUIVerb proxy_menu_verbs [] = {
    BONOBO_UI_VERB ("EditProxies", display_properties_dialog),
    BONOBO_UI_VERB ("AboutDialog", display_about_dialog),
    BONOBO_UI_VERB_END
};

static void
profile_selected_cb(GtkWidget * widget, ProxyApplet *self) 
{
    proxy_properties *proxy;

    proxy = g_object_get_data(G_OBJECT(widget), "profilename" );

    if(proxy) {
	self->properties.useproxy = TRUE;
	self->properties.selected = proxy->name;
    } else 
	self->properties.useproxy = FALSE;

    set_selected_profile( self );

}


static void
add_profile_to_list( proxy_properties *proxy, ProxyApplet *self ) 
{
    GtkWidget *menuitem;
  
    if(proxy) {
	menuitem = gtk_menu_item_new_with_label( proxy->name );
    } else
	menuitem = gtk_menu_item_new_with_label( "No Proxy" );

    g_signal_connect(G_OBJECT(menuitem), "activate",
		     G_CALLBACK(profile_selected_cb), self);

    g_object_set_data(G_OBJECT(menuitem), "profilename",
		      proxy);

    gtk_menu_shell_append ( GTK_MENU_SHELL(self->prof_menu),
			    menuitem );
    gtk_widget_show(menuitem);
}

static void
popup_profile_list (GtkWidget *evbox, gpointer foo,  ProxyApplet *self)
{

    self->prof_menu = gtk_menu_new ();
  
    add_profile_to_list(NULL, self);

    self->properties.proxies = 
	g_list_first (self->properties.proxies); 
    g_list_foreach( self->properties.proxies,
		    (GFunc)add_profile_to_list,
		    self );

    gtk_menu_popup ( GTK_MENU(self->prof_menu),
		     NULL,
		     NULL,
		     NULL,
		     NULL,
		     0,
		     gtk_get_current_event_time() );

}

static void
applet_destroy (GtkWidget *applet, ProxyApplet *self)
{
    g_message (_("destroy proxy-applet"));
    
    /* XXX free other stuff */
    g_free (self);
}


void
panel_changed_size (PanelApplet *panelapplet,
		    gint size,
		    gpointer user_data) 
{
    ProxyApplet *self = user_data;

    gtk_widget_destroy( self->pixmap );
 
    self->pixmap = gnome_pixmap_new_from_file_at_size (PROXY_PIXMAP,
						       size,size );
    gtk_widget_show (self->pixmap);
    gtk_container_add (GTK_CONTAINER (self->evbox), self->pixmap);
}

static void
create_applet_widgets(GtkWidget *widget, ProxyApplet *self)
{
    
    gint h;
    
    h = panel_applet_get_size(self->applet);
    self->pixmap = gnome_pixmap_new_from_file_at_size (PROXY_PIXMAP,
						       h,h);
    g_signal_connect(G_OBJECT(self->applet), "change_size", 
		     G_CALLBACK (panel_changed_size), self );

    gtk_widget_show (self->pixmap);
    self->evbox = gtk_event_box_new();
  
    gtk_container_add (GTK_CONTAINER (self->evbox), self->pixmap);
    gtk_container_add (GTK_CONTAINER (widget), self->evbox);
  
    gtk_signal_connect (GTK_OBJECT (self->evbox),
			"button_release_event",
			GTK_SIGNAL_FUNC (popup_profile_list), 
			self);

    gtk_tree_view_set_headers_visible( GTK_TREE_VIEW(self->proflist), TRUE);
    gtk_tree_view_insert_column ( GTK_TREE_VIEW(self->proflist),
				  gtk_tree_view_column_new_with_attributes ("Profiles",
									    gtk_cell_renderer_text_new (),
									    "text", 0,
									    NULL),
				  0 );

    gtk_signal_connect (GTK_OBJECT (widget),"destroy",
			GTK_SIGNAL_FUNC (applet_destroy),self);


    panel_applet_setup_menu (self->applet, 
			     proxy_menu_xml,
			     proxy_menu_verbs,
			     self );
}

static gboolean
proxy_applet_fill (PanelApplet *applet)
{
    ProxyApplet *self = g_new0 (ProxyApplet, 1);
 
    gnome_window_icon_set_default_from_file (PROXY_PIXMAP);
    panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);

    self->applet = applet;
 
    /* initialize glade */
    glade_gnome_init ();
    self->glade_file= GLADEDIR "/proxy-profiles-applet.glade";
    if (!g_file_exists (self->glade_file)) {
	self->glade_file = g_concat_dir_and_file (GLADEDATADIR, "proxy-profiles-applet.glade");
    }
    if (!g_file_exists (self->glade_file)) {
	show_error_dialog (self, _("Cannot find proxy-profiles-applet.glade"));
	return FALSE;
    }

 
    load_properties (self);

    self->xml = glade_xml_new (self->glade_file,NULL,NULL);
  
    self->toplevel = glade_xml_get_widget (self->xml, "toplevel");
    self->editprof = glade_xml_get_widget (self->xml, "editprof");
    self->proflist = glade_xml_get_widget (self->xml, "profile_tree");
    self->profnamediag = glade_xml_get_widget (self->xml, "profnamediag");

    gtk_signal_connect (GTK_OBJECT (glade_xml_get_widget 
				    (self->xml, "profclose_but")),
			"clicked",
			GTK_SIGNAL_FUNC (close_properties_dialog), self);
    gtk_signal_connect (GTK_OBJECT (glade_xml_get_widget 
				    (self->xml, "add_but")),
			"clicked",
			GTK_SIGNAL_FUNC (add_profile_cb), self);
    gtk_signal_connect (GTK_OBJECT (glade_xml_get_widget 
				    (self->xml, "edit_but")),
			"clicked",
			GTK_SIGNAL_FUNC (edit_profile_cb), self);
    gtk_signal_connect (GTK_OBJECT (glade_xml_get_widget 
				    (self->xml, "del_but")),
			"clicked",
			GTK_SIGNAL_FUNC (delete_profile_cb), self);

    gtk_signal_connect (GTK_OBJECT (glade_xml_get_widget 
				    (self->xml, "apply_button")),
			"clicked",
			GTK_SIGNAL_FUNC (apply_profile_edit_cb), self); 
    gtk_signal_connect (GTK_OBJECT (glade_xml_get_widget 
				    (self->xml, "cancel_button")),
			"clicked",
			GTK_SIGNAL_FUNC (cancel_profile_edit_cb), self);
    gtk_signal_connect ( GTK_OBJECT (self->profnamediag),
			 "response",
			 G_CALLBACK (gtk_widget_hide),
			 GTK_OBJECT (self->profnamediag));
    


    create_applet_widgets(GTK_WIDGET(applet), self);
	
    gtk_widget_show_all(GTK_WIDGET (applet));
    gtk_widget_hide (self->profnamediag);

    return TRUE;
} 

static gboolean
proxy_applet_factory ( PanelApplet *applet,
                       const gchar *iid,
                       gpointer     data)
{
    gboolean retval = FALSE;

    if (!strcmp (iid, "OAFIID:GNOME_Panel_ProxyProfilesApplet"))
	retval = proxy_applet_fill (applet);
  
    return retval;
}


PANEL_APPLET_BONOBO_FACTORY( "OAFIID:GNOME_Panel_ProxyProfilesApplet_Factory",
                             PANEL_TYPE_APPLET,
                             "Proxy-Profiles-Applet",
                             VERSION,
                             (PanelAppletFactoryCallback) proxy_applet_factory,
                             NULL);


